@@ -6,10 +6,12 @@ USTファイルとデータを扱うモジュールです。
 import re
 from collections import UserDict
 from copy import deepcopy
-# from functools import lru_cache
+from os.path import join
 # from pprint import pprint
 from typing import List
 
+from .utau import utau_appdata_root, utau_root
+
 NOTENUM_TO_NOTENAME_DICT = {
     12: 'C0', 13: 'Db0', 14: 'D0', 15: 'Eb0', 16: 'E0', 17: 'F0',
     18: 'Gb0', 19: 'G0', 20: 'Ab0', 21: 'A0', 22: 'Bb0', 23: 'B0',
@@ -53,7 +55,6 @@ NOTENUM_TO_NOTENAME_DICT = {
     '126': 'Gb9', '127': 'G9'
 }
 
-# TODO: シャープに対応する。
 NOTENAME_TO_NOTENUM_DICT = {
     'Cb0': 11, 'C0': 12, 'C#0': 13,
     'Db0': 13, 'D0': 14, 'D#0': 15,
@@ -137,13 +138,14 @@ NOTENAME_TO_NOTENUM_DICT = {
 
 def notenum_as_abc(notenum) -> str:
     """
-    音階番号をABC表記に変更する(C1=24, C4=)
+    音階番号をABC表記に変更する(C1=24, C4=60)
     """
     # raise DeprecationWarning(
     #     '関数 utaupy.ust.notenum_as_abc() は utaupy.utils.notenum2notename に名称変更しました。')
     return NOTENUM_TO_NOTENAME_DICT[int(notenum)]
 
-def load(path: str, encoding: str = 'shift-jis'):
+
+def load(path: str, encoding: str = 'cp932'):
     """
     USTを読み取り
     """
@@ -161,8 +163,11 @@ class Ust:
         super().__init__()
         # ノート(クラスオブジェクト)からなるリスト
         self.version = None   # [#VERSION]
-        self.setting = Note(tag='[#SETTING]')  # [#SETTING]
         self.notes: List[Note] = []  # [#1234], [#INSERT], [#DELETE]
+        self.setting = Note(tag='[#SETTING]')  # [#SETTING]
+        self.setting['Tempo'] = 120
+        del self.setting['Length']
+        del self.setting['NoteNum']
         self.trackend = Note(tag='[#TRACKEND]')  # [#TRACKEND]
         self.next_note = None  # [#NEXT]
         self.previous_note = None  # [#PREV]
@@ -187,7 +192,7 @@ class Ust:
             s = '\n'.join((str_version, s))
         return s
 
-    def load(self, path: str, encoding='shift-jis'):
+    def load(self, path: str, encoding='cp932'):
         """
         ファイルからインスタンス生成
         """
@@ -256,6 +261,20 @@ class Ust:
         # self.notes[0].tempo = tempo
         self.reload_tempo()
 
+    @property
+    def voicedir(self):
+        """
+        UTAU音源のフォルダのパスを返す
+        """
+        voicedir = self.setting['VoiceDir']
+        voicedir = voicedir.replace('%VOICE%', f'{utau_root()}\\voice')
+        voicedir = voicedir.replace('%DATA%', utau_appdata_root())
+        return voicedir
+
+    @voicedir.setter
+    def voicedir(self, path):
+        self.setting['VoiceDir'] = path.strip('\'"')
+
     def clean_tempo(self):
         """
         ローカルテンポが不要な部分を削除する。
@@ -278,6 +297,13 @@ class Ust:
         if 'Tempo' in self.notes[0]:
             self.setting['Tempo'] = self.notes[0]['Tempo']
         current_tempo = self.setting['Tempo']
+
+        # [#PREV]にalternative_tempoを登録
+        previous_note = self.previous_note
+        if previous_note is not None:
+            previous_note.alternative_tempo = previous_note.get('Tempo', current_tempo)
+
+        # 通常のノートにalternative_tempoを登録
         for note in self.notes:
             if 'Tempo' in note:
                 if float(note['Tempo']) == current_tempo:
@@ -286,6 +312,12 @@ class Ust:
                     current_tempo = note['Tempo']
             # current_tempo = note.get('Tempo', current_tempo)
             note.alternative_tempo = float(current_tempo)
+
+        # [#NEXT]にalternative_tempoを登録
+        next_note = self.next_note
+        if next_note is not None:
+            next_note.alternative_tempo = next_note.get('Tempo', current_tempo)
+
         self.clean_tempo()
 
     def reload_tag_number(self, start: int = 0):
@@ -299,19 +331,25 @@ class Ust:
 
     # ノート一括編集系関数ここから----------------------------------------------
     def replace_lyrics(self, before: str, after: str):
-        """歌詞を一括置換（文字列指定・破壊的処理）"""
+        """
+        歌詞を一括置換（文字列指定・破壊的処理）
+        """
         for note in self.notes:
             note.lyric = note.lyric.replace(before, after)
 
     def translate_lyrics(self, before, after):
-        """歌詞を一括置換（複数文字指定・破壊的処理）"""
+        """
+        歌詞を一括置換（複数文字指定・破壊的処理）
+        """
         for note in self.notes:
             note.lyric = note.lyric.translate(before, after)
 
     def vcv2cv(self):
-        """歌詞を平仮名連続音から単独音にする"""
+        """
+        歌詞を平仮名連続音から単独音にする
+        """
         for note in self.notes:
-            note.lyric = note.lyric.split()[-1]
+            note.lyric = note.lyric.split(' ', 1)[-1]
     # ノート一括編集系関数ここまで----------------------------------------------
 
     def insert_note(self, i: int):
@@ -321,7 +359,7 @@ class Ust:
         編集するために、挿入したノートを返す。
         """
         note = Note()
-        note.tag = '[#INSERT]'
+        # note.tag = '[#INSERT]'
         self.notes.insert(i, note)
         return note
 
@@ -332,17 +370,19 @@ class Ust:
         self.notes[i].tag = '[#DELETE]'
 
     def make_finalnote_R(self):
-        """Ustの最後のノートが休符 になるようにする"""
+        """
+        Ustの最後のノートが休符 になるようにする
+        """
         last_note = self.notes[-1]
         # Ust内の最後はTRACKENDなので後ろから2番目のノートで判定
         if last_note.lyric not in ('pau', 'sil', 'R'):
-            print('  末尾に休符を自動追加します。')
             rest_note = deepcopy(last_note)
             rest_note.lyric = 'R'
+            rest_note.length = 480
             self.notes.append(rest_note)
         self.reload_tempo()
 
-    def write(self, path: str, mode: str = 'w', encoding: str = 'shift-jis') -> str:
+    def write(self, path: str, mode: str = 'w', encoding: str = 'cp932') -> str:
         """
         USTをファイル出力
         """
@@ -364,49 +404,56 @@ class Ust:
 
 
 class Note(UserDict):
-    """UST内のノート"""
+    """
+    UST内のノート
+    """
 
-    def __init__(self, tag: str = '[#UNDEFINED]'):
+    def __init__(self, tag: str = '[#INSERT]'):
         super().__init__()
         self['Tag'] = tag
         self.alternative_tempo = None
+        self.length = 480
+        self.notenum = 60
 
     def __str__(self):
         lines = [self['Tag']] + [f'{k}={v}' for (k, v) in self.items() if k != 'Tag']
         return '\n'.join(lines)
 
     @property
-    def tag(self):
-        """タグを確認"""
+    def tag(self) -> str:
+        """
+        ノート識別用のタグ
+        """
         return self['Tag']
 
     @tag.setter
     def tag(self, s):
-        """タグを上書き"""
         self['Tag'] = s
 
     @property
-    def length(self):
-        """ノート長を確認[Ticks]"""
+    def length(self) -> int:
+        """
+        ノート長[Ticks]
+        """
         return int(self['Length'])
 
     @length.setter
     def length(self, x):
-        """ノート長を上書き[Ticks]"""
         self['Length'] = str(x)
 
     @property
     def length_ms(self) -> float:
-        """ノート長を確認[ms]"""
+        """
+        ノート長[ms]
+        """
         return 125 * float(self['Length']) / self.tempo
 
     @length_ms.setter
     def length_ms(self, x):
-        """ノート長を上書き[ms]"""
         self['Length'] = str(round(x * self.tempo / 125))
 
     @property
-    def lyric(self):
+    def lyric(self) -> str:
         """
         歌詞
         """
@@ -417,7 +464,7 @@ class Note(UserDict):
         self['Lyric'] = x
 
     @property
-    def notenum(self):
+    def notenum(self) -> int:
         """
         音階番号
         """
@@ -428,14 +475,14 @@ class Note(UserDict):
         self['NoteNum'] = str(x)
 
     @property
-    def notename(self):
+    def notename(self) -> str:
         """
         音名（C4などの表記）
         """
         return NOTENUM_TO_NOTENAME_DICT[self.notenum]
 
     @notename.setter
-    def notename(self, notename):
+    def notename(self, notename: str):
         self.notenum = NOTENAME_TO_NOTENUM_DICT[str(notename)]
 
     @property
@@ -471,7 +518,7 @@ class Note(UserDict):
         self['PBS'] = str_pbs
 
     @property
-    def pbw(self) -> list:
+    def pbw(self) -> List[int]:
         """
         PBW (mode2ピッチ点の間隔[ms]) を取得
         例) PBW=77,163
@@ -484,7 +531,7 @@ class Note(UserDict):
         return l_pbw
 
     @pbw.setter
-    def pbw(self, list_pbw):
+    def pbw(self, list_pbw: List[int]):
         # リストを整数の文字列に変換
         str_pbw = ','.join(list(map(str, map(int, list_pbw))))
         self['PBW'] = str_pbw
@@ -509,10 +556,10 @@ class Note(UserDict):
         self['PBY'] = str_pby
 
     @property
-    def pbm(self) -> list:
+    def pbm(self) -> List[str]:
         """
         PBM (mode2ピッチ点の形状) を取得
-        例) PBY=-10.6,0.0
+        例) PBY=,,,,
         """
         # 辞書には文字列で登録してある
         s_pby = self['PBM']
@@ -522,13 +569,13 @@ class Note(UserDict):
         return l_pbm
 
     @pbm.setter
-    def pbm(self, list_pbm):
+    def pbm(self, list_pbm: list):
         # リストを文字列に変換
         str_pbm = ','.join(list_pbm)
         self['PBM'] = str_pbm
 
     @property
-    def velocity(self):
+    def velocity(self) -> int:
         """
         子音速度
         """
@@ -561,8 +608,8 @@ class Note(UserDict):
     def label(self, label: str):
         self['Label'] = str(label)
 
-
     # ここからノート操作系-----------------------------------------------------
+
     def delete(self):
         """選択ノートを削除"""
         self.tag = '[#DELETE]'
@@ -582,7 +629,9 @@ class Note(UserDict):
         self.tag = '[#DELETE]\n[#INSERT]'
 
     def suppin(self):
-        """ノートの情報を最小限にする"""
+        """
+        ノートの情報を最小限にする
+        """
         new_data = {}
         new_data['Tag'] = '[#DELETE]\n[#INSERT]'
         new_data['Lyric'] = self.lyric
@@ -593,7 +642,9 @@ class Note(UserDict):
 
 
 def main():
-    """実行されたときの挙動"""
+    """
+    実行されたときの挙動
+    """
     print('デフォ子かわいいよデフォ子\n')
 
     print('ust読み取りテストをします。')
